# Copyright (C) 2008-2012 Bernd Holleczek
#                         Saarland Cancer Registry
#                         b.holleczek@gbe-ekr.saarland.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See file LICENSE for more details.
#
#

plot.period <- function(x, curve=c("absolute", "relative"), cumulative=TRUE, type="o", lty=1, lwd=2, ylim=c(0, 100),
  main="", ylab=paste(ifelse(cumulative, "cumulative", "follow-up year specific"), curve, "survival"),
  xlab="year of follow-up", k=NULL, ...) {

  #check, whether x is an object of class 'period'
  class(x)!="period" && stop("argument ", deparse(substitute(x)), ": not a legitimate object of class period.object.")

  #check and coerce k
  if (!is.null(k)) {
    any(is.na(suppressWarnings(k <- as.integer(k)))) && stop(gettextf("argument k=%s: unable to coerce to integer.", deparse(substitute(k))))
    (length(k)>0) && (k <- k[1]) && is.character(warning(gettextf("argument k=%s: length > 1: first element used.", deparse(substitute(k)), call.=TRUE, immediate.=TRUE)))
  } else {
    k <- length(x$abs.surv)
  }

  #match argument curve against candidate values
  curve <- match.arg(curve)
  mode(cumulative)!="logical" && (cumulative <- TRUE)

  #set variables to be passed to generic plot function
  y <- x[[paste(ifelse(cumulative, "", "cond."), substr(curve, 1, 3), ".surv", sep="")]]
  if (cumulative) {
    y <- c(100, y)
  }
  xlim <- c(ifelse(cumulative, 0, 1), k+0.05)

  #par(mar=c(4.5, 4, 2.5, 1))
  par(mgp=c(2.5, 0.75, 0))

  plot(x=c(ifelse(cumulative, 0, 1):k), y, type=type, lty=lty, lwd=lwd, xlab=xlab, ylab=ylab, main=main,
    xlim=xlim, ylim=ylim, yaxt="n", xaxt="n", xaxs = "i", yaxs = "i", frame.plot=FALSE, cex.lab=1, ...)

  axis(1, at=c(0:k),labels=c(0:k), las=1, mgp=c(2, 0.75, 0), cex=1, cex.axis=0.9)
  axis(2, at=0:5*20,labels=0:5*20, las=2, cex.axis=0.9)

  invisible()
}

