# Copyright (C) 2008-2012 Bernd Holleczek
#                         Saarland Cancer Registry
#                         b.holleczek@gbe-ekr.saarland.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See file LICENSE for more details.
#
#

print.period <- function(x, digits.surv=1, digits.err=1, intermediate.results=FALSE, ...) {

  #check, whether x is an object of type 'period'
  (class(x)!="period") && stop("argument ", deparse(substitute(x)), ": not a legitimate object of class period.object.")

  #output components of x to console
  cat("Call: ", deparse(x$call, width.cutoff=80), ".\n\n", sep="")
  cat(paste("Package version: ", x$version, ".\n\n", sep=""))
  cat(paste("Estimates based on ", sum(x$observations), " observations.\n\n", sep=""))

  if (x$age.adjusted & intermediate.results) {
    cat("Age categories:\n\n")
    temp_t <- data.frame(label=dimnames(x$person.years)[[3]], observations=x$observations, perc=round(prop.table(x$observations)*100, 1), weight=x$w)
    print(temp_t)
    rm(temp_t)
    cat("\n\n")
  }

  cat("Estimates:\n\n")
  estimates <- data.frame("abs"=round(x$abs.surv, digits.surv),
    "se.abs"=round(x$abs.surv.stderr, digits.err),
    "cond.abs"=round(x$cond.abs.surv, digits.surv),
    "se.cond.abs"=round(x$cond.abs.surv.stderr, digits.err),
    "rel"=round(x$rel.surv, digits.surv),
    "se.rel"=round(x$rel.surv.stderr, digits.err),
    "cond.rel"=round(x$cond.rel.surv, digits.surv),
    "se.cond.rel"=round(x$cond.rel.surv.stderr, digits.err))
  print(estimates)
  cat("\n\n")

  cat("abs/rel: cumulative absolute/relative survival\n")
  cat("se.abs/se.rel: standard error of cumulative survival estimates\n")
  cat("cond.abs/cond.rel: follow-up year specific absolute/relative survival\n")
  cat("se.cond.abs/se.cond.rel: standard error of follow-up year specific survival estimates\n\n")
  cat("---\n\n")

  if (intermediate.results) {
    cat("Summed up person years and deaths for each follow-up year, calendar year\n")
    cat("and age category:\n")
    cat("\n\nObserved person years:", "\n")
    for (i in 1:dim(x$person.years)[3]) {
      cat("\n", dimnames(x$person.years)[[3]][i], "\n")
      print(x$person.years[,,i])
    }
    cat("\n\nObserved deaths:", "\n")
    for (i in 1:dim(x$obs.deaths)[3]) {
      cat("\n", dimnames(x$obs.deaths)[[3]][i], "\n")
      print(x$obs.deaths[,,i])
    }
    cat("\n\nExpected deaths:", "\n")
    for (i in 1:dim(x$exp.deaths)[3]) {
      cat("\n", dimnames(x$exp.deaths)[[3]][i], "\n")
      print(x$exp.deaths[,,i])
    }
    if (!is.null(x$exp.person.years)) {
      cat("\n\nExpected person years:", "\n")
      for (i in 1:dim(x$exp.person.years)[3]) {
        cat("\n", dimnames(x$exp.person.years)[[3]][i], "\n")
        print(x$exp.person.years[,,i])
      }
    }
    cat("\n\n")
  }
  invisible(x)
}
 
 

