# Copyright (C) 2008-2012 Bernd Holleczek
#                         Saarland Cancer Registry
#                         b.holleczek@gbe-ekr.saarland.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See file LICENSE for more details.
#
#

write.period <- function(x, file="", quote=TRUE, sep=";", dec=",") {

  #check, whether x is an object of type 'period'
  (class(x)!="period") && stop("argument ", deparse(substitute(x)), ": not a legitimate object of class period.object.")


  #either double quotes or empty string
  quotes <- ifelse(quote, "\"", "")
  
  #write call used to produce argument x
  #double quotes are replaced by single quotes
  cat(paste(quotes, "Call: ", gsub(pattern="\"", replacement="'",
    x=deparse(x$call, width.cutoff=500)), quotes, "\n\n", sep=""), file=file)
  cat(paste(quotes, "Package version:", x$version, quotes, "\n", sep=""), file=file, append=TRUE)
    
  cat(paste(quotes, "Estimates based on ", sum(x$observations), " observations.", quotes, "\n\n",
    sep=""), file=file, append=TRUE)

  if (x$age.adjusted) {
    cat(paste(quotes, "Age categories:", quotes, "\n", sep=""), file=file, append=TRUE)
    temp_t <- data.frame(label=dimnames(x$person.years)[[3]], observations=x$observations, perc=prop.table(x$observations)*100, weight=x$w)

    suppressWarnings(
      write.table(temp_t, file=file, dec=dec, quote=quote, sep=sep, col.names=NA,
        row.names=TRUE, append=TRUE)
      )
    cat("\n\n", file=file, append=TRUE)
    rm(temp_t)
  }

  #write survival estimates and summed up person years and deaths to file
  cat(paste(quotes, "Estimates:", quotes, "\n", sep=""), file=file, append=TRUE)
  suppressWarnings(
    write.table(data.frame("abs"=x$abs.surv, "se.abs"=x$abs.surv.stderr, "cond.abs"=x$cond.abs.surv,
      "se.cond.abs"=x$cond.abs.surv.stderr, "rel"=x$rel.surv, "se.rel"=x$rel.surv.stderr,
      "cond.rel"=x$cond.rel.surv, "se.cond.rel"=x$cond.rel.surv.stderr), file=file, dec=dec,
      sep=sep, quote=quote, col.names=NA, row.names=TRUE, append=TRUE)
    )
  cat("\n\n", file=file, append=TRUE)

  cat(paste(quotes, "Summed up person years and deaths for each follow-up year, calendar year", quotes, "\n", sep=""), file=file, append=TRUE)
  cat(paste(quotes, "and age category:", quotes, "\n", sep=""), file=file, append=TRUE)


  cat(paste("\n", quotes, "Observed person years:", quotes, "\n", sep=""), file=file, append=TRUE)
  for (i in 1:dim(x$person.years)[3]) {
    cat(dimnames(x$person.years)[[3]][i], "\n", file=file, append=TRUE)
    suppressWarnings(
      write.table(x$person.years[,,i], file=file, sep=sep, quote=quote, dec=dec, col.names=NA,
        row.names=TRUE, append=TRUE)
    )
    cat("\n", file=file, append=TRUE)
  }
  
  cat(paste("\n", quotes, "Observed deaths:", quotes, "\n", sep=""), file=file, append=TRUE)
  for (i in 1:dim(x$obs.deaths)[3]) {
    cat(dimnames(x$obs.deaths)[[3]][i], "\n", file=file, append=TRUE)
    suppressWarnings(
      write.table(x$obs.deaths[,,i], file=file, dec=dec, quote=quote, sep=sep, col.names=NA,
        row.names=TRUE, append=TRUE)
    )
    cat("\n", file=file, append=TRUE)
  }
  
  cat(paste("\n", quotes, "Expected deaths:", quotes, "\n", sep=""), file=file, append=TRUE)
  for (i in 1:dim(x$exp.deaths)[3]) {
    cat(dimnames(x$exp.deaths)[[3]][i], "\n", file=file, append=TRUE)
    suppressWarnings(
      write.table(x$exp.deaths[,,i], file=file, dec=dec, quote=quote, sep=sep, col.names=NA,
        row.names=TRUE, append=TRUE)
    )
    cat("\n", file=file, append=TRUE)
  }

  if(!is.null(x$exp.person.years)) {
    cat(paste("\n", quotes, "Expected person years:", quotes, "\n", sep=""), file=file, append=TRUE)
    for (i in 1:dim(x$exp.person.years)[3]) {
      cat(dimnames(x$exp.person.years)[[3]][i], "\n", file=file, append=TRUE)
      suppressWarnings(
        write.table(x$exp.person.years[,,i], file=file, dec=dec, quote=quote, sep=sep, col.names=NA,
          row.names=TRUE, append=TRUE)
      )
      cat("\n\n", file=file, append=TRUE)
    }
  }

  invisible()
}
