\encoding{latin1}
\name{lines.period}
\alias{lines.period}
\title{
  Adds a Curve of Absolute or Relative Survival to an Existing
  Plot
}
\description{
  Adds a curve of absolute or relative survival (either follow-up
  year specific or cumulative estimates) to a current plot
  generated with \code{\link{plot.period}}.
}
\usage{
  \method{lines}{period}(x, curve=c("absolute", "relative"),
      cumulative=TRUE, type="o", lty=2, lwd=2, ...)
}
\arguments{
  \item{x}{an object of class \code{\link{period.object}}.}
  
  \item{curve}{whether \code{absolute} (default) or
    \code{relative} survival estimates should be plotted
    (character vector).}
    
  \item{cumulative}{if \code{TRUE} then cumulative otherwise
    follow-up year specific survival will be plotted (logical).}
    
  \item{type}{type of curve (character).  By default, the
    curve consists of overplotted line and points.}
    
  \item{lty}{line type (numeric). By default, a dashed line is
    plotted.}
    
  \item{lwd}{line width (numeric). By default, a line of width 2
    is plotted.}

  \item{\dots}{other graphical parameters.}
}

\section{Side effects}{A curve is added to the current plot.}

\author{Bernd Holleczek}

\seealso{\code{\link{period}}, \code{\link{hybridperiod}},
  \code{\link{plot.period}}, \code{\link{period.object}}.}

\examples{
## load data of stomach cancer patients

data(stomach)


## load age-specific survival probabilities of males and females
## in Germany in 1996-2005

data(surv.probs.males)
data(surv.probs.females)


## categorize patients according to their age at diagnosis
##
## agr=0 for patients younger than 65 years
## agr=1 for patients 65 years and older

sto.agr <- data.frame(stomach,
    agr=findInterval(stomach$diagage, 65))


## compute age group-specific 5-year survival based on survival
## observed during the calendar period 2001 to 2005 (Ederer II
## method is used for relative survival)

period.result.agr0 <- period(subset(sto.agr, agr==0), 5,
     surv.probs.males, surv.probs.females, 2001, 2005,
     method="edererII")

period.result.agr1 <- period(subset(sto.agr, agr==1), 5,
    surv.probs.males, surv.probs.females, 2001, 2005,
    method="edererII")


## plot conditional relative survival

plot(period.result.agr0, curve="relative", cumulative=TRUE,
    main="Relative Survival of Patients with Invasive Stomach Cancer
    in Saarland (Period Estimate 2001-2005)")
    
lines(period.result.agr1, curve="relative", cumulative=TRUE)

## add legend

legend("bottomleft", c("age at diagnosis <65 years", "age >=65 years"),
    lty = c(1:2), lwd=2, bty="n")

}
\keyword{survival}
