\encoding{latin1}
\name{period.object}
\alias{period.object}
\title{Class Containing Results of a Period Analysis}
\description{
  An object of class \code{period.object} is a list which is
  returned by the functions \code{\link{period}} and
  \code{\link{hybridperiod}}.  It contains follow-up
  year-specific and cumulative estimates of absolute and relative
  survival and corresponding standard errors, summed up person
  years and deaths as well as further details about the analysis
  performed.

  If the data did not provide sufficient information to estimate
  survival for the entire follow-up period, \code{NA} will be
  returned as value of missing estimates.

  Class specific methods exist for the generic functions
  \code{\link{print}}, \code{\link{plot}} and
  \code{\link{lines}}. The \code{\link{write.period}} method may
  be used to write results to a text file for further use with
  spreadsheet programs (e.g. Microsoft Excel, OpenOffice.org Calc).
}
\value{
  The following components are included in a legitimate object of
  class \code{\link{period.object}}:
  
  \item{observations}{number(s) of patients contributing to the
    estimation (numeric vector; stratified by age category,
    if age-adjusted survival was computed).}
    
  \item{abs.surv}{estimated cumulative absolute survival
    proportions (numeric vector).}

  \item{abs.surv.stderr}{standard errors of cumulative absolute
    survival estimates (numeric vector).}
    
  \item{cond.abs.surv}{estimated follow-up year-specific absolute
    survival proportions (numeric vector).}
    
  \item{cond.abs.surv.stderr}{standard error of conditional
    absolute survival estimates (numeric vector).}
    
  \item{rel.surv}{estimated cumulative relative survival (ratio of
    observed and expected survival multiplied by 100; numeric
    vector).}
    
  \item{rel.surv.stderr}{standard error of cumulative relative
    survival (numeric vector).}
    
  \item{cond.rel.surv}{estimated follow-up year specific relative
    survival (ratio of observed and expected survival multiplied
    by 100; numeric vector).}
    
  \item{cond.rel.surv.stderr}{standard error of follow-up
    year-specific relative survival estimate (numeric vector).}
    
  \item{person.years}{person years summed up by follow-up year
    and calendar year of the period of analysis as well as age
    category, if age-standardized survival was computed (three dimensional
    numeric array;  dimensions are: follow-up year (referenced
    with first subscript; \code{k} classes), calendar year of
    period (second subscript; number of classes equals to
    duration of period) and age category (third subscript;
    number of classes equal to number of age categories (one
    class if crude survival was estimated).  E. g.
    \code{person.years[,,1]} references the array of person years
    of the overall cohort or youngest age category, respectively)).}
    
  \item{obs.deaths}{observed deaths summed up by follow-up year,
    calendar year of the period of analysis and age category (see
    \code{person.years} further for details).}

  \item{exp.deaths}{expected deaths summed up by follow-up year,
    calendar year of period of analysis and age category (see
    \code{person.years} for further details).}
    
  \item{exp.person.years}{expected person years summed up by
    follow-up year, calendar year of period of analysis and
    age category (only provided if \sQuote{Hakulinen} method
    was used for relative survival, otherwise \code{NULL};
    see \code{person.years} for further details).}
    
  \item{w}{weights used for estimation of age-standardized
    survival using the \sQuote{Brenner} approach (only provided,
    if age-adjustment was done, otherwise \code{NULL}; see
    \code{\link{period}} for further details).}
    
  \item{call}{the call that produced the object (character
    vector).}
    
  \item{version}{the version of the software package used
    (character vector).}
}
\author{Bernd Holleczek}

\seealso{
  \code{\link{period}}, \code{\link{hybridperiod}},
  \code{\link{print.period}}, \code{\link{write.period}},
  \code{\link{plot.period}}, \code{\link{lines.period}}.
}
\keyword{survival}
