\encoding{latin1}
\name{plot.period}
\alias{plot.period}
\title{
  Plots a Curve of Absolute or Relative Survival
}
\description{
  Produces a plot with a curve of absolute or relative survival
  (either follow-up year-specific or cumulative estimates).
}
\usage{\method{plot}{period}(x, curve=c("absolute", "relative"),
    cumulative=TRUE, type="o", lty=1, lwd=2, ylim=c(0, 100),
    main="", ylab=paste(ifelse(cumulative, "cumulative",
    "follow-up year specific"), curve, "survival"),
    xlab="year of follow-up", k=NULL, ...)
}
\arguments{\item{x}{an object of class
    \code{\link{period.object}}.}
  \item{curve}{whether \code{absolute} (default) or
    \code{relative} survival should be plotted (character
    vector).}
  \item{cumulative}{if \code{TRUE} (default) then cumulative
    otherwise follow-up year-specific survival will be plotted
    (logical).}
  \item{type}{type of the curve (character). By default, the curve
    consists of overplotted line and points.}
  \item{lty}{line type (numeric).  By default, a solid line is
    plotted.}
  \item{lwd}{line width (numeric).  By default, a line width 2 is
    used.}
  \item{ylim}{vertical limits of the plot (numeric vector). By
    default, the y-scale ranges from 0 to 100.}
  \item{main}{title of the plot (character vector).}
  \item{ylab}{label for the y-axis (character vector).}
  \item{xlab}{label for the x-axis (character vector).}
  \item{k}{follow-up time in years (integer).  By default, the follow-up
    time is derived from argument \code{x}.}
  \item{\dots}{other graphical parameters}
}
\details{
  Argument \code{k} may be provided to specify the limit of the x-axis of the plot.
}

\author{Bernd Holleczek}
\seealso{\code{\link{period}}, \code{\link{hybridperiod}},
  \code{\link{lines.period}}, \code{\link{period.object}}.
}
\examples{

## load data of stomach cancer patients

data(stomach)


## load age-specific survival probabilities of males and females
## in Germany in 1996-2005

data(surv.probs.males)
data(surv.probs.females)

## compute period survival estimates for calendar period 2001 to
## 2005 (Ederer II method is used for relative survival)

period.result <- period(stomach, 5, surv.probs.males,
    surv.probs.females, 2001, 2005, method="edererII")


## plot a curve of the estimated relative survival

plot(period.result, curve="absolute",
    main="Observed Survival of Stomach Cancer Patients in
    Saarland (calendar period 2001-05)")
}

\keyword{survival}

