\encoding{latin1}
\name{print.period}
\alias{print.period}
\title{Prints Results of a Period Analysis}
\description{Prints results of a period analysis (follow-up
  year-specific and cumulative absolute and relative survival
  estimates and corresponding standard errors).  Additional
  information on the numbers of observations/patients
  contributing to the estimation, the version of package used and
  the call that produced the \code{\link{period.object}} are
  provided.  Optionally, intermediate results (summed up person
  years and deaths) may be displayed as well.
}
\usage{\method{print}{period}(x, digits.surv=1, digits.err=1,
    intermediate.results=FALSE, ...)
}
\arguments{\item{x}{an object of class
    \code{\link{period.object}}.}
  \item{digits.surv}{decimals of survival estimates (numeric).  By
    default, one decimal place is printed.}
  \item{digits.err}{decimals of standard errors (numeric).  By
    default, one decimal place is printed.}
  \item{intermediate.results}{if \code{TRUE}, summed up person
    years and deaths are printed as well (logical). See
    \code{\link{period.object}} for further details.  By default,
    no intermediate results are printed.}
  \item{...}{other unused arguments}
}
\value{\code{x}, with the invisible flag set.}
\author{Bernd Holleczek}
\seealso{\code{\link{period.object}}, \code{\link{write.period}},
  \code{\link{period}}, \code{\link{hybridperiod}}
}
\examples{
## load data of stomach cancer patients

data(stomach)


## load age-specific survival probabilities of males and females
## in Germany in 1996-2005

data(surv.probs.males)
data(surv.probs.females)


## compute period survival estimates for calendar period 2001 to
## 2005 (Ederer II method is used for relative survival)

period.result <- period(stomach, 5, surv.probs.males,
    surv.probs.females, 2001, 2005, method="edererII")


## print results

print(period.result, intermediate.results=TRUE)
}
\keyword{survival}
