\encoding{latin1}
\name{write.period}
\alias{write.period}
\title{Writes Results of a Period Analysis to a Text File}
\description{
  Writes results of a period analysis (follow-up year-specific
  and cumulative absolute and relative survival estimates and
  corresponding standard errors), summed up person years and
  deaths (for each combination of follow-up year, calendar year
  of the specified analysis period as well as age category, if
  age-adjusted survival was computed), numbers of patients
  contributing to the estimation, the version of the package used
  and the call that produced the \code{\link{period.object}} to a
  text file, which may be used with a spreadsheet program (e.g.
  Microsoft Excel, OpenOffice.org Calc).
}
\usage{write.period(x, file="", quote=TRUE, sep=";", dec=",")}
\arguments{
  \item{x}{an object of class \code{\link{period.object}}.}
  
  \item{file}{file name (character vector). The empty string
    produces output to the console only.}

  \item{quote}{if \code{TRUE} (default), any character vector
    (e. g. row and column names of tables, the call that produced
    the \code{x} object) will be surrounded by double quotes
    (logical).}

  \item{sep}{field separator used in tables of survival estimates
    and person years and deaths (character). By default, a semicolon
    is used.}

  \item{dec}{decimal point (character).  By default, a comma is
    used.}
}
\details{
  The output is similar to the output of method
  \code{\link{print.period}}.  Real numbers (survival estimates
  and their standard errors) are written to the maximal possible
  precision.  By default, a comma is used for decimal points and
  a semicolon as field separator (the convention for csv-files
  in some Western European locales) and character vectors, row
  and column names of tables are double quoted.
}

\author{Bernd Holleczek}
\seealso{\code{\link{period}}, \code{\link{hybridperiod}},
  \code{\link{period.object}}, \code{\link{print.period}}
}
\examples{
## load data of stomach cancer patients

data(stomach)


## load age-specific survival probabilities of males and females
## in Germany in 1996-2005

data(surv.probs.males)
data(surv.probs.females)

## compute period survival estimates for calendar period 2001 to
## 2005 (the EdererII method is used for relative survival)

period.res <- period(stomach, 5, surv.probs.males,
    surv.probs.females, 2001, 2005, method="edererII")

## write results console

write.period(period.res, file="", quote=FALSE)
}

\keyword{survival}
