// Copyright (C) 2008-2010 Bernd Holleczek
//                         Saarland Cancer Registry
//                         b.holleczek@gbe-ekr.saarland.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See file LICENSE for more details.
//
//





	#ifndef M_PI
	#define VAR_OBSERVATIONS "observations"
	#define VAR_PERSON_YEARS "person.years"
	#define VAR_OBSERVED_DEATHS "obs.deaths"
	#define VAR_EXPECTED_DEATHS "exp.deaths"
	#define VAR_EXPECTED_PERSON_YEARS "exp.person.years"
	#define VAR_PERSON_YEARS_BY_AGR "person.years.by.agr"
	#define VAR_CENSORINGS "censorings"
	#endif

	#include <R.h>

	extern "C" {

	#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) dgettext ("periodR", String)
	#else
	#define _(String) (String)
	#endif

	#include <Rinternals.h>
	#include <Rmath.h>
	#include <R_ext/Boolean.h>
	#include <R_ext/Constants.h>
	#include <R_ext/Utils.h>
	#include <R_ext/PrtUtil.h>
	#include <R_ext/Rdynload.h>

	SEXP period(SEXP diagage, SEXP sex, SEXP dy, SEXP dm, SEXP fy, SEXP fm,
				SEXP vitstat, SEXP k, SEXP surv_m, SEXP surv_f, SEXP perbeg, SEXP perend, SEXP agr, SEXP n_agr);

	SEXP periodh(SEXP diagage, SEXP sex, SEXP dy, SEXP dm, SEXP fy, SEXP fm,
				SEXP vitstat, SEXP k, SEXP surv_m, SEXP surv_f, SEXP perbeg, SEXP perend, SEXP agr, SEXP n_agr);

	SEXP hperiod(SEXP diagage, SEXP sex, SEXP dy, SEXP dm, SEXP fy, SEXP fm,
				SEXP vitstat, SEXP k, SEXP surv_m, SEXP surv_f, SEXP diagbeg, SEXP perbeg, SEXP perend, SEXP agr, SEXP n_agr);



	R_CallMethodDef callMethods[] = {
    {"period_C", (DL_FUNC)&period, 14},
    {"periodh_C", (DL_FUNC)&periodh, 14},
    {"hperiod_C", (DL_FUNC)&hperiod, 15},
    {NULL, NULL, 0}
	};


	void R_init_periodR(DllInfo *info)
	{
		R_registerRoutines(info, NULL, callMethods, NULL, NULL);
	}



	void R_unload_periodR(DllInfo *info)
	{
	}






  //definition of period (Ederer II method), periodh (Hakulinen method), hperiod (Ederer II method)
	//compute tables of observed person years, observed and expected deaths
	//rows:       follow up years
	//columns:    calendar years of period
	//
  SEXP period(SEXP diagage, SEXP sex, SEXP dy, SEXP dm, SEXP fy, SEXP fm,
				SEXP vitstat, SEXP k, SEXP surv_m, SEXP surv_f, SEXP perbeg, SEXP perend, SEXP agr, SEXP n_agr)
	{

		//DECLARATIONS
		int n;
		//n := number of observations

    //int n_agr;
    //n_agr := number of age categories

    int len_surv_m, len_surv_f;
    //length of vectors with survival probabilities

		int i;
		//i := control variable for follow-up years
		//variable j is defined locally
		//j := control variable for observations

 		int cy, age;
 		// cy := calendar year
 		// age := (current) age of an individual

		double gp;
		//gp := expected 1-year survival probability of an individual

		SEXP l_array, d_array, e_array, observations;
 		//three dimensional arrays of effective numbers of persons at risk ([i,,]: follow up years;
 		//[,i,]: calendar years of period, [,,i]: age category), observed and expected numbers of deaths


		//COERXION OF PARAMETERS
		diagage = coerceVector(diagage, INTSXP);
		//diagage := age at diagnosis time (n observations)

		sex = coerceVector(sex, INTSXP);
		//sex := sex (n observations)

		dm = coerceVector(dm, INTSXP);
		dy = coerceVector(dy, INTSXP);
		// dm, dy := month and year of diagnosis (n observations)

		fm = coerceVector(fm, INTSXP);
		fy = coerceVector(fy, INTSXP);
		// fm, fy := month and year of end of follow-up (n observations)

		vitstat = coerceVector(vitstat, INTSXP);
		// vitstat := vitalstatus at end of follow-up  (n observations)

		k = coerceVector(k, INTSXP);
		//k := length of follow up

		perbeg = coerceVector(perbeg, INTSXP);
		//perbeg := beginn of period

		perend = coerceVector(perend, INTSXP);
		//perend := end of period

		surv_m = coerceVector(surv_m, VECSXP);
		surv_f = coerceVector(surv_f, VECSXP);
		//surv_m, surv_f := lists of vectors of type double with conditional age-specific survial probabilities of periods
		//specified by column names (e. g. if surv_m contains 2 columns named "1990" and "1995" then column 1 contains survival probabilities
		//for males for calendar years 1990, 1991, 1992, 1993, 1994 and column 2 from 1995 onwards


    //determine length of vectors with survival probabilities
    len_surv_m = length(VECTOR_ELT(surv_m, 0));
    len_surv_f = length(VECTOR_ELT(surv_f, 0));


    agr = coerceVector(agr, INTSXP);
    //agr := agegroup of observation i at the begining of follow up
    //if no age standardized is done, value of agr is always 1 otherwise
    //it consists of integers of value 1, ..., number of age categories

    n_agr = coerceVector(n_agr, INTSXP);


		//INITIALIZATION OF VARIABLES
		//get number of observations
		n = length(diagage);

    //get number of age categories
    int number_agr;
    number_agr = INTEGER(n_agr)[0];

    //duration of follow up period
		int duration_follow_up;
    duration_follow_up = INTEGER(k)[0];

    //duration of calendar period
		int duration_period;
		duration_period = INTEGER(perend)[0]-INTEGER(perbeg)[0] + 1;




		//calendar years for which age-specific 1-year survival probabilities
		//of the general population are available for males and females
		const int surv_m_cols = length(surv_m);
		double* surv_m_col_names = new double[surv_m_cols];
		SEXP R_elem_names = getAttrib(surv_m, R_NamesSymbol);


		int* surv_m_cols_index = new int[surv_m_cols];
		for (int x = 0; x < surv_m_cols; ++x) {
			surv_m_col_names[x] = atof(CHAR(STRING_ELT(R_elem_names, x)));
			surv_m_cols_index[x] = x;
		}
		rsort_with_index(surv_m_col_names, surv_m_cols_index, surv_m_cols);
		R_CheckUserInterrupt();


		const int surv_f_cols = length(surv_f);
    double* surv_f_col_names = new double[surv_f_cols];
		R_elem_names = getAttrib(surv_m, R_NamesSymbol);

		int* surv_f_cols_index = new int[surv_f_cols];
		for (int x = 0; x < surv_f_cols; ++x) {
			surv_f_col_names[x] = atof(CHAR(STRING_ELT(R_elem_names, x)));
			surv_f_cols_index[x] = x;
		}
		rsort_with_index(surv_f_col_names, surv_f_cols_index, surv_f_cols);
		R_CheckUserInterrupt();


		//COMPUTE SURVIVAL
		int * mflag = new int;
		int interval;


		bool * contribObs = new bool[n];
		for (int j = 0; j < n; ++j) {
			contribObs[j] = false;
		}
		R_CheckUserInterrupt();


		//l_array := threedimensional array of observed numbers of person years for each combination
		//of follow up year, calendar year and age category
		//l_vector := stores observed person time for each patient
    PROTECT(l_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
    double* l_vector = new double[duration_follow_up * duration_period * number_agr];

		//d_array := array of observed numbers of deaths for each combination
		//of follow up year, calendar year and age category
		//d_vector := stores observed death for each patient
		PROTECT (d_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* d_vector = new double[duration_follow_up * duration_period * number_agr];

		//e_array := array of expected numbers of deaths (from population life tables for each
		//combination of follow up year, calendar year and age category
		//e_vector :=  stores exptected numbers of deaths for each patient
		PROTECT(e_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* e_vector = new double[duration_follow_up * duration_period * number_agr];


		//initialize arrays and vectors
		int curr_pos;
		for(int i=0; i<INTEGER(k)[0]; ++i) {
			for(int y=0; y<duration_period; ++y) {
        for (int z=0; z<number_agr; ++z) {
				  curr_pos = i + y*duration_follow_up + z*duration_follow_up*duration_period;
				  REAL(l_array)[curr_pos] = REAL(d_array)[curr_pos] = REAL(e_array)[curr_pos] = 0;
				  l_vector[curr_pos] = d_vector[curr_pos] = e_vector[curr_pos] = 0;
				}
			}
		}
		R_CheckUserInterrupt();

    int curr_calendar_year;
 		//determine the contribution of each observation to persons at risk (perl)
		//observed deaths (perd) and expected deaths in the absence of cancer (perde)
		//for each follow up year i (1<=i<=k).

		for(int j=0; j<n; ++j) {


      for(cy=INTEGER(perbeg)[0]; cy<=INTEGER(perend)[0]; ++cy) {

        //increment calendar year each time cy is incremented;
        //curr_calendar_year takes values from 0 onwards;
        curr_calendar_year = cy-INTEGER(perbeg)[0];

  			i = cy-INTEGER(dy)[j];

 				//i:= numerical difference between current calendar year of period and year of diagnosis
 				//take observed survival into account if i lies within follow-up interval (0<=i<=k)
				//and current calendar year <= end of follow up
				if (0<=i && i<=duration_follow_up && cy <= INTEGER(fy)[j]) {

					gp = interval = -1;
					age = INTEGER(diagage)[j] + i;
					*mflag = 0;

					//here, get conditional survival probabilities for current individual (age, sex and calendar year)
					//gp := expected 1-year survival probability of an individual of the same sex
					//and age in the general population

					if (INTEGER(sex)[j] == 1) {
					    //use survival probability of age len_surv_m - 1 for all individuals of age len_surv_m and more
					    if (age > len_surv_m - 1) {
  							interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
	   						gp = (*mflag>=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[len_surv_m-1] : -1;
					    } else if (age == 0) {
		  					interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
			   				gp = (*mflag>=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[0] : -1;
					    } else {
				  			interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
					   		gp = (*mflag>=0) ? (REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[age-1] + REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[age])/2 : -1;
					    }

					} else if (INTEGER(sex)[j] == 2) {
              //use survival probability of age len_surv_f - 1 for all individuals of age len_surv_f and more
					    if (age > len_surv_f - 1) {
						  	interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							  gp = (*mflag>=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[len_surv_f-1] : -1;
					    } else if (age == 0) {
  							interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
	   						gp = (*mflag>=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[0] : -1;
					    } else {
		  					interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
			   				gp = (*mflag>=0) ? (REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[age-1] + REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[age])/2 : -1;
					    }
					}

					if (gp < 0) error(_("not possible to determine survival probability for observation %d.\n"), j+1);

					// contribution of each individual to person time at risk (l_array), observed deaths (d_array) and expected
					// deaths (e_array) is determined for each the follow-up year



					//sum observed and expected person years in follow-up year i and i+1
					if (i > 0) {
            l_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up * duration_period * (INTEGER(agr)[j]-1)] += 0.5;
						e_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up * duration_period * (INTEGER(agr)[j]-1)] += 0.5 * (1 - gp);
					  contribObs[j] = true;
          }
					if (((INTEGER(vitstat)[j]==1 && cy<(INTEGER(fy)[j]-1)) || (INTEGER(vitstat)[j]==2 && cy<INTEGER(fy)[j]) || INTEGER(fm)[j]>=INTEGER(dm)[j]) && i<duration_follow_up) {
						l_vector[(i) + duration_follow_up*curr_calendar_year + duration_follow_up * duration_period * (INTEGER(agr)[j]-1)] += 0.5;
						e_vector[(i) + duration_follow_up*curr_calendar_year + duration_follow_up * duration_period * (INTEGER(agr)[j]-1)] += 0.5 * (1 - gp);
					  contribObs[j] = true;
					}


					//bugfix June 21, 2011
					if (INTEGER(vitstat)[j]==1 && cy==(INTEGER(fy)[j]-1) && INTEGER(fy)[j]==(INTEGER(perend)[0]+1) && INTEGER(fm)[j]<INTEGER(dm)[j] && i<INTEGER(k)[0] ) {
            l_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
            e_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * (1 - gp);
						contribObs[j] = true;
          }
          //end bugfix


          //sum observed deaths
          //calendar month of dead < calendar month of diagnosis -> death in follow up year i
					if (INTEGER(vitstat)[j]==2 && INTEGER(fm)[j]<INTEGER(dm)[j] && i>0 && INTEGER(fy)[j]==cy) {
						d_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up * duration_period * (INTEGER(agr)[j]-1)] = 1;
  					contribObs[j] = true;
					}

  				//sum observed and expected person years and observed deaths
 					//in follow-up year i+1
					if (INTEGER(vitstat)[j]==2 && INTEGER(fm)[j]>=INTEGER(dm)[j] && INTEGER(fy)[j]==cy && i<duration_follow_up) {
						l_vector[(i) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
						e_vector[(i) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = (1 - gp);
						d_vector[(i) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
					  contribObs[j] = true;
					}

				}

        R_CheckUserInterrupt();
			}

      //add values from l_vector_temp, d_array_temp, e_array_temp to l_array, d_array and e_array
      for (int i=0; i < duration_follow_up; ++i) {
        for(int y=0; y < duration_period; ++y) {
          for (int z=0; z < number_agr; ++z) {
            curr_pos = i + y*duration_follow_up + z * duration_follow_up * duration_period;
            REAL(l_array)[curr_pos] += l_vector[curr_pos];
            REAL(d_array)[curr_pos] += d_vector[curr_pos];
            REAL(e_array)[curr_pos] += e_vector[curr_pos];
            l_vector[curr_pos] = d_vector[curr_pos] = e_vector[curr_pos] = 0;
          }
        }
      }
      R_CheckUserInterrupt();
		}


		//return number of observations in each age category
		//used for survival estimates
		PROTECT(observations = allocVector(INTSXP, number_agr));
		for (int z=0; z < number_agr; ++z) {
		  INTEGER(observations)[z] = 0;
		}

		for (int j=0; j < n; ++j) {
			if (contribObs[j]) {
			  ++INTEGER(observations)[INTEGER(agr)[j]-1];
			}
		}

    R_CheckUserInterrupt();

		//set up results
		//function modperiodh returns a list containing
		// - array of person time at risk
		// - array of observed deaths
		// - array of expected deaths
		// - array of expected person years
		//for each year of follow up and calendar year of period
		//number of observations contributing for estimates

		SEXP results, results_names;
		PROTECT(results = allocVector(VECSXP, 5));
		PROTECT(results_names = allocVector(STRSXP, 5));

		SET_VECTOR_ELT(results, 0, l_array);
		SET_VECTOR_ELT(results, 1, d_array);
		SET_VECTOR_ELT(results, 2, e_array);
		SET_VECTOR_ELT(results, 3, R_NilValue);
		SET_VECTOR_ELT(results, 4, observations);

		SET_STRING_ELT(results_names, 0, mkChar(VAR_PERSON_YEARS));
		SET_STRING_ELT(results_names, 1, mkChar(VAR_OBSERVED_DEATHS));
		SET_STRING_ELT(results_names, 2, mkChar(VAR_EXPECTED_DEATHS));
		SET_STRING_ELT(results_names, 3, mkChar(VAR_EXPECTED_PERSON_YEARS));
		SET_STRING_ELT(results_names, 4, mkChar(VAR_OBSERVATIONS));

		setAttrib(results, R_NamesSymbol, results_names);
		UNPROTECT(6);
		return(results);

	}
	//end of period(...)




	SEXP periodh(SEXP diagage, SEXP sex, SEXP dy, SEXP dm, SEXP fy, SEXP fm,
				SEXP vitstat, SEXP k, SEXP surv_m, SEXP surv_f, SEXP perbeg, SEXP perend, SEXP agr, SEXP n_agr)
	{

		//DECLARATIONS
		int n;
		//n := number of observations

    //int n_agr;
    //n_agr := number of age categories

    int len_surv_m, len_surv_f;
    //length of vectors with survival probabilities

		int i;
		//i := control variable for follow-up years
		//variable j is defined locally
		//j := control variable for observations

 		int cy, age;
 		// cy := calendar year
 		// age := (current) age of an individual

		double gp;
		//gp := expected 1-year survival probability of an individual

 		SEXP l_array, d_array, e_array, le_array, observations;
 		//arrays for effective numbers of persons at risk (rows: follow up years; cols: calendar years of period)
 		//observed deaths (rows: follow up years; cols: calendar years of period), expected numbers of deaths
		//(rows: follow up years; cols: calendar years of period) and expected numbers of person years/persons at
		//risk  (rows: follow up years; cols: calendar years of period)

		int agej;

		double wt, cp;

		//COERXION OF PARAMETERS

		diagage = coerceVector(diagage, INTSXP);
		//diagage := age at diagnosis time (n observations)

		sex = coerceVector(sex, INTSXP);
		//sex := sex (n observations)

		dm = coerceVector(dm, INTSXP);
		dy = coerceVector(dy, INTSXP);
		// dm, dy := month and year of diagnosis (n observations)

		fm = coerceVector(fm, INTSXP);
		fy = coerceVector(fy, INTSXP);
		// fm, fy := month and year of end of follow-up (n observations)

		vitstat = coerceVector(vitstat, INTSXP);
		// vitstat := vitalstatus at end of follow-up  (n observations)

		k = coerceVector(k, INTSXP);
		//k := length of follow up

		perbeg = coerceVector(perbeg, INTSXP);
		//perbeg := beginn of period

		perend = coerceVector(perend, INTSXP);
		//perend := end of period

		surv_m = coerceVector(surv_m, VECSXP);
		surv_f = coerceVector(surv_f, VECSXP);
		//surv_m, surv_f := lists of vectors of type double with conditional age-specific survial probabilities of periods
		//specified by column names (e. g. if surv_m contains 2 columns named "1990" and "1995" then column 1 contains survival probabilities
		//for males for calendar years 1990, 1991, 1992, 1993, 1994 and column 2 from 1995 onwards

    //determine length of vectors with survival probabilities
    len_surv_m = length(VECTOR_ELT(surv_m, 0));
    len_surv_f = length(VECTOR_ELT(surv_f, 0));


    agr = coerceVector(agr, INTSXP);
    //agr := agegroup of observation i at the begining of follow up
    //if no age standardized is done, value of agr is always 1 otherwise
    //it consists of integers of value 1, ..., number of age categories

    n_agr = coerceVector(n_agr, INTSXP);

		//INITIALIZATION OF VARIABLES
		n = length(diagage);

    //get number of age categories
    int number_agr;
    number_agr = INTEGER(n_agr)[0];

    //n_agr = 1;
    //for (int j=0; j<n; ++j) {
    //  n_agr = imax2(n_agr, INTEGER(agr)[j]);
    //}

		int duration_follow_up;
    duration_follow_up = INTEGER(k)[0];

		int duration_period;
		duration_period = INTEGER(perend)[0]-INTEGER(perbeg)[0] + 1;

		//l_array := threedimensional array of observed numbers of person years for each combination
		//of follow up year, calendar year and age category
		//l_vector := stores observed person time for each patient
    PROTECT(l_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
    double* l_vector = new double[duration_follow_up * duration_period * number_agr];

		//d_array := array of observed numbers of deaths for each combination
		//of follow up year, calendar year and age category
		//d_vector := stores observed death for each patient
		PROTECT (d_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* d_vector = new double[duration_follow_up * duration_period * number_agr];

		//e_array := array of expected numbers of deaths (from population life tables for each
		//combination of follow up year, calendar year and age category
		//e_vector :=  stores exptected numbers of deaths for each patient
		PROTECT(e_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* e_vector = new double[duration_follow_up * duration_period * number_agr];


		//e_array => array of expected numbers of deaths (from population life
		//tables for each combination of follow up year and calendar year
		//e_array_temp => stors exptected numbers of deaths for each patient
		PROTECT(le_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* le_vector = new double[duration_follow_up * duration_period * number_agr];


		//initialize two dimensional arrays
		int curr_pos;
		for (int i=0; i<duration_follow_up; ++i) {
			for (int y=0; y<duration_period; ++y) {
			  for (int z=0; z<number_agr; ++z) {
          curr_pos = i + y * duration_follow_up + z * duration_follow_up * duration_period;
          REAL(l_array)[curr_pos] = REAL(d_array)[curr_pos] = REAL(e_array)[curr_pos] = REAL(le_array)[curr_pos] = 0;
          l_vector[curr_pos] = d_vector[curr_pos] = e_vector[curr_pos] = le_vector[curr_pos] = 0;
			  }
			}
		}
		R_CheckUserInterrupt();




		//calendar years for which survival data of the general population
		//is available for males and females
		const int surv_m_cols = length(surv_m);
		double* surv_m_col_names = new double[surv_m_cols];
		SEXP R_elem_names = getAttrib(surv_m, R_NamesSymbol);


		int* surv_m_cols_index = new int[surv_m_cols];
		for (int x = 0; x < surv_m_cols; ++x) {
			surv_m_col_names[x] = atof(CHAR(STRING_ELT(R_elem_names, x)));
			surv_m_cols_index[x] = x;
			R_CheckUserInterrupt();
		}
		rsort_with_index(surv_m_col_names, surv_m_cols_index, surv_m_cols);

		const int surv_f_cols = length(surv_f);
		double* surv_f_col_names = new double[surv_f_cols];
		R_elem_names = getAttrib(surv_m, R_NamesSymbol);

		int* surv_f_cols_index = new int[surv_f_cols];
		for (int x = 0; x < surv_f_cols; ++x) {
			surv_f_col_names[x] = atof(CHAR(STRING_ELT(R_elem_names, x)));
			surv_f_cols_index[x] = x;
			R_CheckUserInterrupt();
		}
		rsort_with_index(surv_f_col_names, surv_f_cols_index, surv_f_cols);


		//COMPUTE SURVIVAL
		int * mflag = new int;
		int interval;

		bool * contribObs = new bool[n];
		for (int j = 0; j < n; ++j) {
			contribObs[j] = false;
		}

 		int curr_calendar_year;
 		//determine the contribution of each observation to persons at risk (perl)
		//observed deaths (perd) and expected deaths in the absence of cancer (perde)
		//for each follow up year i (1<=i<=k).
		//each of these contributions is weighted by the patient's weight w

		for(int j=0; j<n; ++j) {

		  for(cy=INTEGER(perbeg)[0]; cy<=INTEGER(perend)[0]; ++cy) {

   			//increment calendar year each time cy is incremented;
   			//curr_calendar_year takes values from 0 onwards;
   			curr_calendar_year = cy-INTEGER(perbeg)[0];

        i = cy-INTEGER(dy)[j];

   			//i:= numerical difference between current calendar year of period and year of diagnosis
   			//take observed survival into account if i lies within follow-up interval (0<=i<=k)
  			//and current calendar year <= end of follow up
  			if (0<=i && i<=duration_follow_up && cy <= INTEGER(fy)[j]) {

  				if (i>0) {
  					l_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
  					contribObs[j] = true;
  				}

  				if (((INTEGER(vitstat)[j]==1 && cy<(INTEGER(fy)[j]-1)) || (INTEGER(vitstat)[j]==2 && cy<INTEGER(fy)[j]) || (INTEGER(fm)[j]>=INTEGER(dm)[j])) && i<duration_follow_up) {
  					l_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
  					contribObs[j] = true;
  				}

					//bugfix June 21, 2011
					if (INTEGER(vitstat)[j]==1 && cy==(INTEGER(fy)[j]-1) && INTEGER(fy)[j]==(INTEGER(perend)[0]+1) && INTEGER(fm)[j]<INTEGER(dm)[j] && i<INTEGER(k)[0] ) {
            l_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
						contribObs[j] = true;
          }
          //end bugfix


  				if (INTEGER(vitstat)[j]==2 && (INTEGER(fm)[j]<INTEGER(dm)[j]) && i>0 && INTEGER(fy)[j]==cy) {
  					d_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
  					contribObs[j] = true;
  				}

  				if (INTEGER(vitstat)[j]==2 && (INTEGER(fm)[j]>=INTEGER(dm)[j]) && INTEGER(fy)[j]==cy && i<duration_follow_up) {
  					l_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
  					d_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
  					contribObs[j] = true;
  				}
  			}

  			if (0 <= i && i<=duration_follow_up) {

  				age = INTEGER(diagage)[j] + i;
  				gp = interval = -1;
  				*mflag = 0;

          if (INTEGER(sex)[j] == 1) {
            if (age > len_surv_m - 1) {
  						interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  						gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[len_surv_m-1] : -1;
            } else if (age == 0) {
  						interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  						gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[0] : -1;
            } else {
  						interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  						gp = (*mflag >=0) ? (REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[age-1] + REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[age])/2 : -1;
            }
          } else if (INTEGER(sex)[j] == 2) {
            if (age > len_surv_f - 1) {
  				  	interval = findInterval(surv_f_col_names, surv_f_cols,  (double) cy, FALSE, FALSE, 0, mflag) - 1;
  						gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[len_surv_f-1] : -1;
            } else if (age == 0) {
  					  interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  						gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[0] : -1;
            } else {
  						interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  						gp = (*mflag >=0) ? (REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[age-1] + REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[age])/2 : -1;
            }
          }

          if (gp < 0) error(_("survival probability for observation %d could not be determined.\n"), j);

          //gp := expected 1-year survival probability of an individual of the same sex
  	      //and age in the general population
  	      //wt := expected cumulative probability of survival from age at diagnosis to current
  	      //age of a person of the same sex and age in the general population

          wt = 1;

          for(int l=INTEGER(dy)[j]+1; l<=cy; ++l) {
  				  agej = INTEGER(diagage)[j] + (l - INTEGER(dy)[j]);
  					*mflag = 0;
  					cp = interval = -1;

  					if (agej>99) {
  						if (INTEGER(sex)[j] == 1) {
   							interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  							cp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[99] : -1;
  						} else if (INTEGER(sex)[j] == 2) {
  							interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  							cp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[99] : -1;
  						}
  					} else {
  						if (INTEGER(sex)[j] == 1) {
   							interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  							cp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[agej-1] : -1;
  						} else if (INTEGER(sex)[j] == 2) {
  							interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
  							cp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[agej-1] : -1;
  						}
  					}

            if (cp < 0) error(_("expected cumulative survival probability for observation %d could not be estimated.\n"), j);
  						wt = wt * cp;
  					}

            if(i==0) {
  						le_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt;
  						e_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt * (1-gp);
  					}
  					if (0<i && i<duration_follow_up) {
  						le_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt;
  						le_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt;
  						e_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt * (1-gp);
  						e_vector[i + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt * (1-gp);
  					}
  					if (0<i && i==duration_follow_up) {
  						le_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt;
  						e_vector[(i-1) + duration_follow_up*curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * wt * (1-gp);
  					}
	       }
 	    }


      //add values from l_vector_temp, d_array_temp, e_array_temp to l_array, d_array and e_array
      for (int i=0; i < duration_follow_up; ++i) {
        for(int y=0; y < duration_period; ++y) {
          for (int z=0; z < number_agr; ++z) {
            curr_pos = i + y*duration_follow_up + z * duration_follow_up * duration_period;
            REAL(l_array)[curr_pos] += l_vector[curr_pos];
            REAL(d_array)[curr_pos] += d_vector[curr_pos];
            REAL(e_array)[curr_pos] += e_vector[curr_pos];
            REAL(le_array)[curr_pos] += le_vector[curr_pos];
            l_vector[curr_pos] = d_vector[curr_pos] = e_vector[curr_pos] = le_vector[curr_pos] = 0;
          }
        }
      }

      R_CheckUserInterrupt();

    }



		//return number of observations in each age category
		//used for survival estimates
		PROTECT(observations = allocVector(INTSXP, number_agr));
		for (int z=0; z < number_agr; ++z) {
		    INTEGER(observations)[z] = 0;
		}

		for (int j=0; j < n; ++j) {
			if (contribObs[j]) {
			    ++INTEGER(observations)[INTEGER(agr)[j]-1];
			}
		}

    R_CheckUserInterrupt();

		//set up results
		//function modperiodh returns a list containing
		// - array of person time at risk
		// - array of observed deaths
		// - array of expected deaths
		// - array of expected person years
		//for each year of follow up and calendar year of period
		//number of observations contributing for estimates

		SEXP results, results_names;
		PROTECT(results = allocVector(VECSXP, 5));
		PROTECT(results_names = allocVector(STRSXP, 5));

		SET_VECTOR_ELT(results, 0, l_array);
		SET_VECTOR_ELT(results, 1, d_array);
		SET_VECTOR_ELT(results, 2, e_array);
		SET_VECTOR_ELT(results, 3, le_array);
		SET_VECTOR_ELT(results, 4, observations);

		SET_STRING_ELT(results_names, 0, mkChar(VAR_PERSON_YEARS));
		SET_STRING_ELT(results_names, 1, mkChar(VAR_OBSERVED_DEATHS));
		SET_STRING_ELT(results_names, 2, mkChar(VAR_EXPECTED_DEATHS));
		SET_STRING_ELT(results_names, 3, mkChar(VAR_EXPECTED_PERSON_YEARS));
		SET_STRING_ELT(results_names, 4, mkChar(VAR_OBSERVATIONS));

		setAttrib(results, R_NamesSymbol, results_names);
		UNPROTECT(7);
		return(results);

	}
	//end of periodh(...)




	SEXP hperiod(SEXP diagage, SEXP sex, SEXP dy, SEXP dm, SEXP fy, SEXP fm,
				   		  SEXP vitstat, SEXP k, SEXP surv_m, SEXP surv_f, SEXP diagbeg, SEXP perbeg, SEXP perend, SEXP agr, SEXP n_agr)
	{

    int n;
		//n := number of observations

    //int n_agr;
    //n_agr := number of age categories

    int len_surv_m, len_surv_f;
    //length of vectors with survival probabilities

		int i;
		//i := control variable for follow-up years
    //variable j is defined locally
		//j := control variable for observations

		int cy, age;
		// cy := calendar year
		// age := (current) age of an individual

		double gp;
		//gp := expected 1-year survival probability of an individual

		int beg;
		//beg :=

		SEXP l_array, d_array, e_array, observations;
 		//arrays for effective numbers of persons at risk (rows: follow up years; cols: calendar years of period)
 		//observed deaths (rows: follow up years; cols: calendar years of period) and
		//expected numbers of deaths (rows: follow up years; cols: calendar years of period)


		//coerxion of passed arguments

		diagage = coerceVector(diagage, INTSXP);
		//diagage := age in years at diagnosis time (n observations)

		k = coerceVector(k, INTSXP);
		//k := length of follow up

		sex = coerceVector(sex, INTSXP);
		//sex := sex of individual (n observations

		dy = coerceVector(dy, INTSXP);
		dm = coerceVector(dm, INTSXP);
		//dy, dm := month and year of diagnosis (n observations)

		fy = coerceVector(fy, INTSXP);
		fm = coerceVector(fm, INTSXP);
		//fm, fy := month and year of end of follow up (n observations)

		surv_m = coerceVector(surv_m, VECSXP);
		surv_f = coerceVector(surv_f, VECSXP);
		//surv_m, surv_f := lists of vectors of type double with conditional age-specific survial probabilities of periods
		//specified by column names (e. g. if surv_m contains 2 columns named "1990" and "1995" then column 1 contains survival probabilities
		//for males for calendar years 1990, 1991, 1992, 1993, 1994 and column 2 from 1995 onwards

    //determine length of vectors with survival probabilities
    len_surv_m = length(VECTOR_ELT(surv_m, 0));
    len_surv_f = length(VECTOR_ELT(surv_f, 0));


		vitstat = coerceVector(vitstat, INTSXP);
		//vitstat := vital status at end of follow up (n observations)

		diagbeg = coerceVector(diagbeg, INTSXP);
		//diagbeg := diagnosis year of patients which contribute fully to estimation of survival

		perbeg = coerceVector(perbeg, INTSXP);
		//perbeg := begin of period

		perend = coerceVector(perend, INTSXP);
		//perend := end of period


    agr = coerceVector(agr, INTSXP);
    //agr := agegroup of observation i at the begining of follow up
    //if no age standardized is done, value of agr is always 1 otherwise
    //it consists of integers of value 1, ..., number of age categories

    n_agr = coerceVector(n_agr, INTSXP);

		//some initializations
		n = length(diagage);


    int number_agr;
    number_agr = INTEGER(n_agr)[0];
    //n_agr = 1;
    //for (int j=0; j<n; ++j) {
    //  n_agr = imax2(n_agr, INTEGER(agr)[j]);
    //}


		int duration_follow_up;
    duration_follow_up = INTEGER(k)[0];

		int duration_period;
		duration_period = INTEGER(perend)[0]-INTEGER(diagbeg)[0] + 1;


    //calendar years for which survival data of the general population
		//is available for males and females
		const int surv_m_cols = length(surv_m);
		double* surv_m_col_names = new double[surv_m_cols];
		int* surv_m_cols_index = new int[surv_m_cols];

		SEXP R_elem_names = getAttrib(surv_m, R_NamesSymbol);

		for (int x = 0; x < surv_m_cols; ++x) {
			surv_m_col_names[x] = atof(CHAR(STRING_ELT(R_elem_names, x)));
			surv_m_cols_index[x] = x;
			R_CheckUserInterrupt();
		}
		rsort_with_index(surv_m_col_names, surv_m_cols_index, surv_m_cols);

		const int surv_f_cols = length(surv_f);
		double* surv_f_col_names = new double[surv_f_cols];
		int* surv_f_cols_index = new int[surv_f_cols];

		R_elem_names = getAttrib(surv_f, R_NamesSymbol);

		for (int x = 0; x < surv_f_cols; ++x) {
			surv_f_col_names[x] = atof(CHAR(STRING_ELT(R_elem_names, x)));
			surv_f_cols_index[x] = x;
			R_CheckUserInterrupt();
		}
		rsort_with_index(surv_f_col_names, surv_f_cols_index, surv_f_cols);

		//COMPUTE SURVIVAL
		int * mflag = new int;

		int interval;

		//vector contribObs is used as flag for each observation contribution for estimate
		bool * contribObs = new bool[n];
		for (int j = 0; j < n; ++j) {
			contribObs[j] = false;
		}

		//l_array := threedimensional array of observed numbers of person years for each combination
		//of follow up year, calendar year and age category
		//l_vector := stores observed person time for each patient
    PROTECT(l_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
    double* l_vector = new double[duration_follow_up * duration_period * number_agr];

		//d_array := array of observed numbers of deaths for each combination
		//of follow up year, calendar year and age category
		//d_vector := stores observed death for each patient
		PROTECT (d_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* d_vector = new double[duration_follow_up * duration_period * number_agr];

		//e_array := array of expected numbers of deaths (from population life tables for each
		//combination of follow up year, calendar year and age category
		//e_vector :=  stores exptected numbers of deaths for each patient
		PROTECT(e_array = alloc3DArray(REALSXP, duration_follow_up, duration_period, number_agr));
		double* e_vector = new double[duration_follow_up * duration_period * number_agr];


		//initialize arrays and vectors
		int curr_pos;
		for(int i=0; i<INTEGER(k)[0]; ++i) {
			for(int y=0; y<duration_period; ++y) {
        for (int z=0; z<number_agr; ++z) {
				  curr_pos = i + y*duration_follow_up + z*duration_follow_up*duration_period;
				  REAL(l_array)[curr_pos] = REAL(d_array)[curr_pos] = REAL(e_array)[curr_pos] = 0;
				  l_vector[curr_pos] = d_vector[curr_pos] = e_vector[curr_pos] = 0;
				}
			}
		}
		R_CheckUserInterrupt();

 		int curr_calendar_year;


		//determine the contribution of each observation to persons at risk (perl)
		//observed deaths (perd) and expected deaths in the absence of cancer (perde)
		//for each follow up year i (1<=i<=k).

		for(int j=0; j<n; ++j) {

      //determine relevant starting point of period
      if (INTEGER(dy)[j] < INTEGER(diagbeg)[0]) {
	 			beg = INTEGER(perbeg)[0];
			} else {
				beg = INTEGER(dy)[j];
			}

			for(cy=beg; cy<=INTEGER(perend)[0]; ++cy) {

        //increment calendar year each time cy is incremented;
        //curr_calendar_year takes values from 0 onwards;
        curr_calendar_year = cy - INTEGER(diagbeg)[0];

				i = cy-INTEGER(dy)[j];
				if (0<=i && i<=INTEGER(k)[0] && cy <= INTEGER(fy)[j]) {

					age = INTEGER(diagage)[j] + i;
					*mflag = 0;
					gp = interval = -1;


					if (INTEGER(sex)[j] == 1) {
            if (age > len_surv_m - 1) {
							interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[len_surv_m-1] : -1;
							contribObs[j] = true;
					  } else if (age == 0) {
							interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[0] : -1;
							contribObs[j] = true;
					  } else {
							interval = findInterval(surv_m_col_names, surv_m_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							gp = (*mflag >=0) ? (REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[age-1] + REAL(VECTOR_ELT(surv_m, surv_m_cols_index[interval]))[age])/2 : -1;
							contribObs[j] = true;
					  }
					} else if (INTEGER(sex)[j] == 2) {
            if (age > len_surv_f - 1) {
							interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[len_surv_f-1] : -1;
							contribObs[j] = true;
            } else if (age == 0) {
							interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							gp = (*mflag >=0) ? REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[0] : -1;
							contribObs[j] = true;
            } else {
							interval = findInterval(surv_f_col_names, surv_f_cols, (double) cy, FALSE, FALSE, 0, mflag) - 1;
							gp = (*mflag >=0) ? (REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[age-1] + REAL(VECTOR_ELT(surv_f, surv_f_cols_index[interval]))[age])/2 : -1;
							contribObs[j] = true;
            }
					}


          if (gp < 0) error(_("survival probability for observation %d could not be determined.\n"), j);

          //gp := expected 1-year survival probability of an individual of the same sex
          //and age in the general population
					if (i > 0) {
						l_vector[(i-1) + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
						e_vector[(i-1) + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * (1 - gp);
						contribObs[j] = true;
					}

					if (((INTEGER(vitstat)[j]==1 && cy<INTEGER(fy)[j]-1) || (INTEGER(vitstat)[j]==2 && cy<INTEGER(fy)[j]) || INTEGER(fm)[j]>=INTEGER(dm)[j]) && i<INTEGER(k)[0]) {
						l_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
						e_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * (1 - gp);
						contribObs[j] = true;
					}


					//bugfix June 21, 2011
					if (INTEGER(vitstat)[j]==1 && cy==(INTEGER(fy)[j]-1) && INTEGER(fy)[j]==(INTEGER(perend)[0]+1) && INTEGER(fm)[j]<INTEGER(dm)[j] && i<INTEGER(k)[0] ) {
            l_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5;
            e_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] += 0.5 * (1 - gp);
						contribObs[j] = true;
          }
          //end bugfix

					if (INTEGER(vitstat)[j]==2 && INTEGER(fm)[j]<INTEGER(dm)[j] && i>0 && INTEGER(fy)[j]==cy) {
						d_vector[(i-1) + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
						contribObs[j] = true;
					}

					if (INTEGER(vitstat)[j]==2 && INTEGER(fm)[j]>=INTEGER(dm)[j] && INTEGER(fy)[j]==cy && i<INTEGER(k)[0]) {
					  l_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
						e_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = (1 - gp);
						d_vector[i + duration_follow_up * curr_calendar_year + duration_follow_up*duration_period*(INTEGER(agr)[j]-1)] = 1;
						contribObs[j] = true;
					}
				}
			}

      //add values from l_vector_temp, d_array_temp, e_array_temp to l_array, d_array and e_array
      for (int i=0; i < duration_follow_up; ++i) {
        for(int y=0; y < duration_period; ++y) {
          for (int z=0; z < number_agr; ++z) {
            curr_pos = i + y*duration_follow_up + z * duration_follow_up * duration_period;
            REAL(l_array)[curr_pos] += l_vector[curr_pos];
            REAL(d_array)[curr_pos] += d_vector[curr_pos];
            REAL(e_array)[curr_pos] += e_vector[curr_pos];
            l_vector[curr_pos] = d_vector[curr_pos] = e_vector[curr_pos] = 0;
          }
        }
      }
      R_CheckUserInterrupt();

		}

		//return number of observations in each age category
		//used for survival estimates
		PROTECT(observations = allocVector(INTSXP, number_agr));
		for (int z=0; z < number_agr; ++z) {
      INTEGER(observations)[z] = 0;
		}

		for (int j=0; j < n; ++j) {
			if (contribObs[j]) {
        ++INTEGER(observations)[INTEGER(agr)[j]-1];
			}
		}

    R_CheckUserInterrupt();

		//set up results
		//function modperiodh returns a list containing
		// - array of person time at risk
		// - array of observed deaths
		// - array of expected deaths
		// - array of expected person years
		//for each year of follow up and calendar year of period
		//number of observations contributing for estimates

		SEXP results, results_names;
		PROTECT(results = allocVector(VECSXP, 5));
		PROTECT(results_names = allocVector(STRSXP, 5));

		SET_VECTOR_ELT(results, 0, l_array);
		SET_VECTOR_ELT(results, 1, d_array);
		SET_VECTOR_ELT(results, 2, e_array);
		SET_VECTOR_ELT(results, 3, R_NilValue);
		SET_VECTOR_ELT(results, 4, observations);

		SET_STRING_ELT(results_names, 0, mkChar(VAR_PERSON_YEARS));
		SET_STRING_ELT(results_names, 1, mkChar(VAR_OBSERVED_DEATHS));
		SET_STRING_ELT(results_names, 2, mkChar(VAR_EXPECTED_DEATHS));
		SET_STRING_ELT(results_names, 3, mkChar(VAR_EXPECTED_PERSON_YEARS));
		SET_STRING_ELT(results_names, 4, mkChar(VAR_OBSERVATIONS));

		setAttrib(results, R_NamesSymbol, results_names);
		UNPROTECT(6);
		return(results);

	}
	//end hperiod

}
